(function ($) {
    'use strict';
    $(document).ready(function () {

        let searchCurrentIndex = 0;

        let sourceMergeId, targetMergeId;

        let cacheOptList = {};

        const statusTranslate = {
            Running: 'В процессе',
            Finished: 'Завершен',
            Error: 'Ошибка',
            Waiting: 'В очереди',
            Canceled: 'Отменён',
        }

        function check_markup_range(price_from, price_to) {
            if (parseInt(price_from) <= parseInt(price_to)) {
                return true;
            }
            alert('Стартовая цена должна быть меньше конечной!');
            return false;

        }

        function update_markup(object) {

            if ($(object).is('#config')) return;

            let line = $(object).closest('tr');

            let dealerId = $('#dealers_list').children(':selected').attr('id');
            let token = $('#user_token').val();

            let data = {
                'id': $(line).attr('data-id'),
                'category': $(line).find('.markup_params.categories_tree').val(),
                'price_from': $(line).find('.markup_params.price_from').val(),
                'price_to': $(line).find('.markup_params.price_to').val(),
                'price_markup': $(line).find('.markup_params.price_markup').val(),
                'price_is_rrc': $(line).find('.markup_params.price_is_rrc').prop('checked') ? 1 : 0,
            };

            if (!dealerId || !check_markup_range(data.price_from, data.price_to)) return;

            $.ajax({
                type: 'POST',
                url: 'index.php?route=extension/for_dealer_import/updatePriceMarkup&token=' + token,
                data: data,
                context: data,
                dataType: 'json',
                success: (res) => {

                    if (res.res == 0) {
                        console.debug(res);
                    } else {
                        notifyMsg('Изменения сохранены', 'Изменения в наценках сохранены')
                    }

                }
            });

        }


        function getOptionsList(section_id) {

            if (cacheOptList && cacheOptList[section_id]) {
                return cacheOptList[section_id]
            }

            return new Promise((resolve) => {

                let token = $('#user_token').val();

                let data = {
                    'section_id': section_id,
                    'price_id': $('#dealers_list').children(':selected').attr('id'),
                };

                $.ajax({
                    type: 'POST',
                    url: 'index.php?route=extension/for_dealer_import/getSectionOptionsList&token=' + token,
                    data: data,
                    dataType: 'json',
                    success: (res) => {
                        resolve(res);

                        cacheOptList[section_id] = res;
                    }
                });

            });
        }


        $(document).on('keyup.list focus.list', '.options_merge.option_name, #option_name', async function () {
            let sectionId = $('#option_merge_section').children(':selected').attr('value')

            let list = [];
            let options = await getOptionsList(sectionId);

            options.forEach((item) => {
                list.push(item)
            });

            $(this).autocomplete({source: list, minLength: 0})
        })


        /**
         *
         * Replaces modal
         *
         */

        let lastReplacesBtn = null;

        $(document).on('input', '[data-replace-from], [data-replace-to]', function () {
            onReplacesInputChange();
        });

        $(document).on('click', '[remove-replaces-item]', function () {
            removeReplaceItem($(this).attr('data-replaces-key'));
        });

        $('[data-target="#replacesModal"]').attr('data-toggle', 'modal');

        $('#add-new-replace').on('click', function () {
            addReplacesItem();
        });

        $('[replaces-modal-recover]').find('a').on('click', function () {
            recoverReplacesItems();
        });

        $('[replaces-save-btn]').on('click', function () {
            let id = $(lastReplacesBtn).closest('[replaces-row-id]').attr('replaces-row-id');
            let replaces = saveJsonReplaces();

            saveReplaces(id, replaces);
        });

        $('#auto_load_options').off('change').on('change', function () {
            let dealer_id = $('#dealers_list').children(':selected').attr('id');

            let token = $('#user_token').val();

            let data = {
                'dealer_id': dealer_id,
                'auto_load_options': $(this).prop('checked') ? 1 : 0,
            };

            $('.auto_load_options__wrapper').css('opacity', 0.6);

            $.ajax({
                type: 'POST',
                url: 'index.php?route=extension/for_dealer_import/updatePriceMarkupAutoLoadOptions&token=' + token,
                data: data,
                dataType: 'json',
                success: (res) => {
                    $('.auto_load_options__wrapper').css('opacity', 1);
                    if (res.res == 0) {
                        console.debug(res);
                    } else {
                        notifyMsg('Параметры обновлены', 'Изменения сохранены');
                    }
                }
            });
        });

        $('#option_merge_section').change(function () {
            if ($(this).val() < 1) return;

            let sectionId = $(this).val();

            queryChange('section_id', sectionId);
            location.reload();
        });

        $('.schedule_update_time').on('change', function () {
            var selected = $(this).children(':selected');
            var dealer_id = $(this).data('price-id');
            var token = $('#user_token').val();
            $.ajax({
                type: 'POST',
                url: 'index.php?route=extension/for_dealer_import/updateScheduleDate&token=' + token,
                data: {
                    'dealer_id': dealer_id,
                    'update_hour': selected.val()
                },
                dataType: 'json',
                success: function (res) {
                    if (res.res == 0) {
                        console.debug(res);
                    } else {
                        notifyMsg('Время обновлено', 'Изменения сохранены');
                    }
                }
            });

        });

        $('.schedule_update_type').on('change', function () {
            var selected = $(this).children(':selected');
            var dealer_id = $(this).data('price-id');
            var token = $('#user_token').val();

            $.ajax({
                type: 'POST',
                url: 'index.php?route=extension/for_dealer_import/updateScheduleType&token=' + token,
                data: {
                    'dealer_id': dealer_id,
                    'type': selected.val()
                },
                dataType: 'json',
                success: function (res) {
                    if (res.res == 0) {
                        console.debug(res);
                    } else {
                        notifyMsg('Изменения сохранены', 'Тип обновления по расписанию изменен');
                    }
                }
            });
        });

        $('.checkbox.update_dealer').on('change', function () {
            var ischecked = $(this).is(':checked');
            var token = $('#user_token').val();

            if (ischecked) return;

            if (confirm('Вы уверены, что хотите прервать задачу?')) {
                if ($(this).data('cmd-id')) {
                    $.ajax({
                        type: 'POST',
                        url: 'index.php?route=extension/for_dealer_import/cancelCommand&token=' + token,
                        data: {'cmd_id': $(this).data('cmd-id')},
                        dataType: 'json',
                        success: function (res) {
                            if (res.res == 0) {
                                console.debug(res);
                            } else {
                                alert('Задача отменена!');
                            }
                        }
                    });
                    //alert('uncheckd ' + $(this).val());
                }
            } else {
                $(this).prop('checked', true);
            }

        });

        $('.markup_params').on('change', function () {
            update_markup($(this).parent().parent());
        });

        $('.options_merge').off('change.edit').on('change.edit', function () {
            update_options_merge($(this).closest('[replaces-row-id]'));
        });

        $('#add_price_markup').click(function () {
            let dealer_id = $('#dealers_list_settings').children(':selected').attr('id');
            let token = $('#user_token').val();

            let data = {
                'dealer_id': dealer_id,
                'category': $('#categories_tree').children(':selected').val(),
                'price_from': $('#price_from').val(),
                'price_to': $('#price_to').val(),
                'price_markup': $('#price_markup_category').val(),
                'price_is_rrc': $('#price_is_rrc').prop('checked') ? 1 : 0,
            };

            if (dealer_id != 0 && check_markup_range(data.price_from, data.price_to)) {
                $.ajax({
                    type: 'POST',
                    url: 'index.php?route=extension/for_dealer_import/addPriceMarkup&token=' + token,
                    data: data,
                    context: data,
                    dataType: 'json',
                    success: function (res) {
                        if (res.res == 0) {
                            console.debug(res);
                        } else {
                            let newRow = $('#config').clone();
                            let newId = res.id;

                            $(newRow).attr('data-id', newId).attr('id', '');

                            $(newRow).find('#price_is_rrc').removeAttr('id');

                            $(newRow).find('[data-remove-on-add]').remove();

                            let select = newRow.find('select');

                            select.attr('data-id', newId).attr('id', '');

                            select.val($(this)[0].category);
                            select.addClass('markup_params categories_tree');

                            newRow.find('input').addClass('markup_params');
                            newRow.find('input#price_from').attr('data-id', newId).attr('id', '');

                            newRow.find('input#price_to').attr('data-id', newId).attr('id', '');

                            newRow.find('input#price_markup_category').attr('data-id', newId).attr('id', '');

                            newRow.append(`
                                <td style="text-align: center">
                                    <a alt="f182" class="delete delete_price_markup" data-id="${newId}"></a>
                                </td>
                            `);

                            newRow.insertBefore('#config');

                            $('.markup_params').off('change').on('change', function () {
                                update_markup($(this).parent().parent());
                            });
                        }
                    }
                });
            }
        });

        $('#add_options_merge').click(function () {

            let dealer_id = $('#dealers_list').children(':selected').attr('id');

            let data = {
                'dealer_id': dealer_id,
                'category': $('#option_merge_section').children(':selected').val(),
                'option_name': $('#option_name').val(),
                'changed_option_name': $('#changed_option_name').val()
            };

            let token = $('#user_token').val();

            $.ajax({
                type: 'POST',
                url: 'index.php?route=extension/for_dealer_import/addOptionsMerge&token=' + token,
                data: data,
                dataType: 'json',
                success: (res) => {

                    if (res.res == 0) {
                        return console.debug(res);
                    }

                    let newRow = $('#config-options-merge').clone();
                    let newId = res.id;

                    $(newRow).attr('replaces-row-id', newId).attr('id', '');
                    $(newRow).find('.custom-combobox').remove()
                    $(newRow).find('[data-remove-on-add]').remove();

                    newRow.find('input').addClass('options_merge');

                    newRow.find('input#option_name').addClass('option_name').attr('data-id', newId).attr('id', '');
                    newRow.find('input#changed_option_name').addClass('changed_option_name').attr('data-id', newId).attr('id', '');

                    newRow.append(`
                        <td>
                            <button class="button button-primary" style="margin: 0px 0" data-json-replaces="" data-toggle="modal"
                                    data-target="#replacesModal">Открыть список атозамен
                                    (<span data-count-replaces="${newId}">0</span>)
                            </button>
                        </td>
                        <td style="text-align: center">
                            <a class="cursor dashicons dashicons-trash pointer delete delete_options_merge"
                               data-id="${newId}"></a>
                        </td>
                    `);

                    newRow.insertBefore('#config-options-merge');

                    $('.options_merge').off('change.edit').on('change.edit', function () {
                        update_options_merge($(this).parent().parent());
                    });

                    combobox();
                }
            });

        });

        $('#dealer_cat_import').click(function () {

            var dealer_id = $('#dealers_list').children(':selected').attr('id');
            var token = $('#user_token').val();
            if (dealer_id != 0 && confirm('Вы уверены, что хотите импортировать дерево каталога дистрибьютора в свой сайт?')) {
                $.ajax({
                    type: 'POST',
                    url: 'index.php?route=extension/for_dealer_import/importDealerCategoriesTree&token=' + token,
                    data: {'dealer_id': dealer_id},
                    dataType: 'json',
                    success: function (res) {
                        if (res.res == 0) {
                            console.debug(res);
                        } else {
                            location.reload();
                        }
                    }
                });
            }
        });

        $('#delete_empty_bindings').click(function () {
            var token = $('#user_token').val();


            $('#delete_empty_bindings').text('Очистка...');
            $.ajax({
                type: 'POST',
                url: 'index.php?route=extension/for_dealer_import/deleteEmptyBindings&token=' + token,
                dataType: 'json',
                success: function (res) {
                    if (res.res == 0) {
                        console.debug(res);
                    } else {
                        location.reload();
                    }
                }
            });
        });

        $('.dealers_list').change(function () {
            let dealer_id = $(this).children(':selected').attr('id');

            if (dealer_id != 0) {
                queryChange('dealer_id', dealer_id);

                if ($(this).data('tab') === 'options_merge') {
                    queryChange('section_id', '0');
                }

                location.reload();
            }
        });

        $('#dealer_categories ul li span[data-dealer-id]').on('click', function (event) {
            sourceMergeId = $(this).data('dealer-id');
            if ($(event.target).is('.clear_merge')) return;

            if ($(this).hasClass('merging')) {
                sourceMergeId = null;
                $(this).removeClass('merging');
                return;
            }

            $('#dealer_categories').find('.merging').not('.animated-background').removeClass('merging');
            $(this).addClass('merging');
            tryMerge();

        });

        $('#local_categories ul li span[data-term-id]').on('click', function (event) {
            targetMergeId = $(this).data('term-id');
            if ($(event.target).is('.clear_merge')) return;

            if ($(this).hasClass('merging')) {
                targetMergeId = null;
                $(this).removeClass('merging');
                return;
            }

            $('#local_categories').find('.merging').not('.animated-background').removeClass('merging');
            $(this).addClass('merging');
            tryMerge();
        });


        $('#multiple-create-options').click(function () {
            let sectionId = $('#option_merge_section').val();
            let token = $('#user_token').val();

            if (sectionId == '0') {
                return alert('Выберите существующую категорию (кроме "Все")')
            }

            $(this).text('Запрос отправлен, ожидайте');

            $.ajax({
                type: 'POST',
                url: 'index.php?route=extension/for_dealer_import/multipleCreateOptions&token=' + token,
                data: {
                    section_id: sectionId,
                    price_id: $('#dealers_list').children(':selected').attr('id')
                },
                dataType: 'json',
                success: function (res) {
                    location.reload()
                }
            });

        });


        $(document).on('click', '[data-toggle="modal"]', function () {
            $($(this).attr('data-target')).addClass('d-block');
        });

        $(document).on('click', '[data-dismiss="modal"]', function () {
            $(this).closest('.modal.d-block').removeClass('d-block');
            $('.modal-open').removeClass('modal-open');
            $('.modal-backdrop').remove();
        });


        $(document).on('click', '.clear_merge', function () {
            event.preventDefault();
            var source_id = $(this).data('owner-id');
            var token = $('#user_token').val();
            var data = {'source_id': source_id, 'price_id': $('#dealers_list').children(':selected').attr('id')}

            $(this).addClass('spin-anim');

            $.ajax({
                type: 'POST',
                url: 'index.php?route=extension/for_dealer_import/clearMergedCategories&token=' + token,
                data: data,
                context: {'source_id': source_id, 'object': $(this)},
                dataType: 'json',
                success: function (res) {
                    if (res.res == 0) {
                        console.debug(res);
                    } else {
                        $('#dealer-category-' + $(this)[0].source_id).removeClass('merged');
                        $('#dealer-category-' + $(this)[0].source_id).parent('li').removeClass('merged-li');
                        //$(this)[0].object.remove();
                        $(this)[0].object.parent().html('')
                    }
                }
            });

        });

        $(document).on('click', '#params_update', function () {
            event.preventDefault();

            var dealer_id = $('#dealers_list_settings').children(':selected').attr('id');
            if (dealer_id == 0) {
                return false;
            }

            var token = $('#user_token').val();
            var other_images = $('#other_images').is(':checked') ? 1 : 0;

            var data = {
                dealer_id: dealer_id,
                small_quantity: $('#small_quantity').val(),
                medium_quantity: $('#medium_quantity').val(),
                large_quantity: $('#large_quantity').val(),
                price_markup: $('#price_markup').val(),
                round_price: $('#round_price').val(),
                round_rule: $('#round_rule').val(),
                product_add_table: $('#product_add_table').prop('checked') ? 1 : 0,
                other_images: other_images,
                meta: {
                    partno_in_model: $('#partno_in_model').prop('checked') ? 1 : 0,
                    overwrite_product_name: $('#overwrite_product_name').prop('checked') ? 1 : 0,
                }
            };

            $.ajax({
                type: 'POST',
                url: 'index.php?route=extension/for_dealer_import/updateDealerParams&token=' + token,
                data: data,
                dataType: 'json',
                success: function (res) {
                    if (res.res == 0) {
                        console.debug(res);
                    } else {
                        notifyMsg('Параметры обновлены', 'Изменения сохранены');
                    }
                }
            });

        });

        $(document).on('click', '.delete_price_markup', function () {
            let token = $('#user_token').val();
            let data = {
                'id': $(this).attr('data-id'),
            };

            $(this).addClass('spin-anim');

            $.ajax({
                type: 'POST',
                url: 'index.php?route=extension/for_dealer_import/deletePriceMarkup&token=' + token,
                data: data,
                context: {'object': $(this)},
                dataType: 'json',
                success: function (res) {
                    if (res.res == 0) {
                        console.debug(res);
                    } else {
                        $(this)[0].object.parent().parent().remove();
                    }
                }
            });
        });

        $(document).on('click', '.delete_options_merge', function () {

            $(this).addClass('spin-anim');

            let token = $('#user_token').val();
            let data = {'id': $(this).attr('data-id')};

            $.ajax({
                type: 'POST',
                url: 'index.php?route=extension/for_dealer_import/deleteOptionsMerge&token=' + token,
                data: data,
                context: {'object': $(this)},
                dataType: 'json',
                success: function (res) {
                    if (res.res == 0) {
                        return console.debug(res);
                    }

                    $(`[replaces-row-id="${data.id}"]`).remove();
                }
            });

        });

        $(document).on('click', '[data-json-replaces]', function () {
            buildReplacesModal(this);
            checkUnsavedReplaceChanges(this);
        });


        $('a[data-toggle="tab"]').on('click', function () {
            queryChange('tab', $(this).attr('href').replace(/#tab-/g, ''));
        });


        $('[data-toggle-matched]').click(function () {
            let target = $(this).attr('data-toggle-matched');
            let applied = $(this).attr('data-applied') === 'true';

            toggleMatchedProperties(!applied, '#' + target);
            $(this).attr('data-applied', !applied).text(!applied ? 'Показать все' : 'Сведённые');
        });


        $('[html-search]').off('change keyup').on('change keyup', function (e) {
            let val = $(this).val();
            let options = JSON.parse($(this).attr('html-search-options') || '{}');
            let count = $(options.wrapper).find('.highlight-search').length;

            if (e.keyCode === 13) {
                return $(options.btn).click();
            }

            $(options.wrapper).find('.highlight-search.active').removeClass('active');
            $(options.wrapper).find('.highlight-search').removeClass('highlight-search');
            // let searchVariants = Object.values(searchConverter.do(val));

            if (!val.trim()) {
                $(options.wrapper).scrollTop(0);
                $(options.btn).removeClass('visible').attr('search-index', 0);
                searchCurrentIndex = 0;

                return;
            }

            $(options.wrapper).find(options.item).each(function (i) {
                let flag = false;
                let element = $(this).find('ul').length ? 'null' : this;

                if (`${$(this).text()}`.toLocaleLowerCase().includes(val.toLocaleLowerCase())) {
                    flag = true;
                }

                $(element)[flag ? 'addClass' : 'removeClass']('highlight-search');

                if (i === count) {
                    setTimeout(() => {

                        let highlighted = $(options.wrapper).find('.highlight-search');

                        if (Number($(options.btn).attr('search-index')) >= 0) {
                            searchCurrentIndex = Number($(options.btn).attr('search-index'));
                        } else {
                            searchCurrentIndex = 0;
                        }

                        if (searchCurrentIndex + 1 > $(highlighted).length) {
                            searchCurrentIndex = 0;
                        }

                        let btnText = 'Показан ' + (searchCurrentIndex + 1) + ' из ' + $(highlighted).length;

                        let firstEl = $(highlighted).eq(searchCurrentIndex);

                        $(options.btn).text(btnText).addClass('visible').attr('search-index', searchCurrentIndex);

                        $(firstEl).addClass('active');

                        if (!$(firstEl).length) {
                            $(options.btn).text('Ничего не найдено :(');
                        }

                        if (!$(firstEl).offset()) return;

                        $(options.wrapper).scrollTop(
                            $(options.wrapper).scrollTop() +
                            $(firstEl).offset().top - $(options.wrapper).outerHeight() / 2
                        );

                    });
                }

            });

        });

        $('[html-search-clear]').off('click').on('click', function () {
            $($(this).attr('html-search-navigator')).removeClass('visible');
            $($(this).attr('html-search-clear')).val('');

            setTimeout(() => {
                $($(this).attr('html-search-clear')).trigger('keyup', true);
            }, 100);
        });

        $('[html-search-navigator]').not('[html-search-clear]').off('click').on('click', function () {
            searchCurrentIndex = Number($(this).attr('search-index')) || 0;
            searchCurrentIndex++;
            $(this).attr('search-index', searchCurrentIndex);
            $($(this).attr('html-search-navigator')).trigger('keyup');
        });

        $('[data-href="[page]"]').off('click').on('click', function (e) {

            queryChange($(this).attr('data-param'), $(this).attr('data-page'));

            window.location.reload();

            e.stopPropagation();
            e.preventDefault();
        });


        $('[repeat-job]').on('click', function () {

            let token = $('#user_token').val();
            let jobId = $(this).attr('repeat-job');

            $.ajax({
                type: 'POST',
                url: 'index.php?route=extension/for_dealer_import/updateJobById&token=' + token,
                data: {
                    job_id: jobId,
                    job_data: {status: 4}
                },
                dataType: 'json',
                success: (res) => {
                    $('#job-row-status-' + jobId).find('.status-col').text('В очереди');
                }
            });
        });

        $('[delete-job]').on('click', function () {

            let token = $('#user_token').val();
            let jobId = $(this).attr('delete-job');

            $.ajax({
                type: 'POST',
                url: 'index.php?route=extension/for_dealer_import/deleteJobById&token=' + token,
                data: {
                    job_id: jobId,
                },
                dataType: 'json',
                success: (res) => {
                    $('#job-row-status-' + jobId).remove();
                }
            });
        });

        $('[delete-all-jobs]').on('click', function (e) {

            if (confirm('Вы точно хотите удалить ВСЕ процессы?') !== true) {
                e.stopPropagation();
                return false;
            }

            let token = $('#user_token').val();

            $.ajax({
                type: 'POST',
                url: 'index.php?route=extension/for_dealer_import/deleteJobById&token=' + token,
                data: {},
                dataType: 'json',
                success: (res) => {
                    location.reload()
                }
            });

        });

        $('[delete-finished-jobs]').on('click', function () {

            if (confirm('Вы точно хотите удалить ЗАВЕРШЕННЫЕ процессы?') !== true) {
                e.stopPropagation();
                return false;
            }

            let token = $('#user_token').val();

            $.ajax({
                type: 'POST',
                url: 'index.php?route=extension/for_dealer_import/deleteFinishedJobs&token=' + token,
                data: {},
                dataType: 'json',
                success: (res) => {
                    location.reload()
                }
            });
        });

        $('[name="for_dealer_import_settings_calls_per_iteration"]').on('keyup', function () {
            let value = Number($(this).val());

            if ($(this).val().trim() === '') return;

            $(this).val(value >= 7 ? 7 : (value <= 0 ? 1 : value))
        });

        function toggleMatchedProperties(status, wrapper) {
            if (!status) {
                $(wrapper + '.categories-section').find('li').show();
                return
            }

            $(wrapper + '.categories-section').find('li').each(function () {
                $(this)[($(this).hasClass('merged-li') || $(this).find('ul li.merged-li').length) ? 'show' : 'hide']()
            });
        }


        function update_options_merge(row) {
            if ($(row).is('#config')) return;

            let line = $(row).closest('tr');
            let token = $('#user_token').val();

            let data = {
                'action': 'update_options_merge',
                'id': $(line).attr('replaces-row-id'),
                'category': $('#option_merge_section').val(),
                'option_name': $(line).find('.option_name').val(),
                'changed_option_name': $(line).find('.changed_option_name').val(),
            };

            $.ajax({
                type: 'POST',
                url: 'index.php?route=extension/for_dealer_import/updateOptionsMerge&token=' + token,
                data: data,
                context: data,
                dataType: 'json',
                success: (res) => {

                    if (res.res == 0) {
                        console.debug(res);
                    } else {
                        notifyMsg('Изменения сохранены')
                    }

                }
            });
        }

        function tryMerge() {
            let data = {
                source_id: sourceMergeId,
                target_id: targetMergeId,
                price_id: $('#dealers_list').children(':selected').attr('id')
            }

            if (!sourceMergeId || !targetMergeId) return;

            let token = $('#user_token').val();
            let dealer_category = $('#dealer-category-' + data.source_id);

            $(dealer_category).addClass('animated-background');

            sourceMergeId = targetMergeId = null;

            $.ajax({
                type: 'POST',
                url: 'index.php?route=extension/for_dealer_import/mergeCategories&token=' + token,
                data: data,
                context: data,
                dataType: 'json',
                success: function (res) {
                    if (!res.res) {
                        console.debug(res);
                    } else {
                        const localItem = '#site-term-' + data.target_id;
                        const localItemName = $(localItem).text();

                        const generateMergedHtml = (ownerId) => {
                            return `
                                <span>(${localItemName})</span>
                                <span class="clear_merge" data-owner-id="${ownerId}"></span>`;
                        };

                        /** Убираем анимации */
                        $(localItem).removeClass('merging');
                        $(dealer_category).removeClass('animated-background merging');

                        /** Добавляем класс на общую обёртку для фильтрации "сведенные/все" */
                        $(dealer_category).parent('li').addClass('merged-li');

                        /** Добавляем класс на все внутренние элементы для фильтрации "сведенные/все" */
                        dealer_category.parent().find('li').addClass('merged-li');

                        /** Добавляем крестик для target parent */

                        $(dealer_category).next('span').html(generateMergedHtml(data.source_id));

                        /** Добавляем крестик для каждого children, с учетом его data-dealer-id */
                        dealer_category.parent().find('li.merged-li').each(function () {
                            const ownerId = $(this).find('span[data-dealer-id]').attr('data-dealer-id');

                            $(this).find('span[data-dealer-id]').addClass('merged');

                            $(this).find('span[data-dealer-id]').next('span').html(generateMergedHtml(ownerId));
                        });

                        dealer_category.addClass('merged');
                    }
                }
            });
        }


        $('.run_dealers_update').click(function (event) {
            event.preventDefault();
            var actions = {};
            var i = 0;
            var token = $('#user_token').val();
            $('.update_dealer:checked').each(function (index, value) {
                if (!$(this).attr('data-cmd-id')) {
                    if (actions[$(this).val()] == undefined) {
                        actions[$(this).val()] = [];
                    }
                    actions[$(this).val()].push($(this).data('dealer-id'));
                    //console.log($(this).val());
                }
            });

            if (!$.isEmptyObject(actions)) {

                $(this).text('Запрос отправлен...');

                $.ajax({
                    type: 'POST',
                    url: 'index.php?route=extension/for_dealer_import/addJob&token=' + token,
                    data: {'cmd_list': actions},
                    dataType: 'json',
                    success: (res) => {

                        $(this).text('Обновить выбранных дистрибьюторов');
                        $(this).text('Обновить выбранных дистрибьюторов');

                        if (res.res == 0) {
                            console.debug(res);
                        } else {
                            $.each(res.data, function (key, value) {
                                //console.log(key);
                                $.each(value, function (price_id, cmd_id) {
                                    //console.log('Price_id: '+price_id+'   Cmd_id: '+cmd_id);
                                    $('#' + key + '-' + price_id).attr('data-cmd-id', cmd_id);
                                    $('#' + key + '_records-' + price_id).attr('data-cmd-id', cmd_id);
                                    $('#' + key + '_check-' + price_id).attr('data-cmd-id', cmd_id);

                                    //console.log($('#'+key+'-'+price_id).data('cmd-id'));
                                })
                            });
                        }
                    }
                });
            }
        });

        function check_jobs_status() {
            var cmd_id_list = [];
            $('.update_dealer:checked').each(function (index, value) {
                if ($(this).attr('data-cmd-id')) {
                    cmd_id_list.push($(this).attr('data-cmd-id'));
                }
            });
            var token = $('#user_token').val();
            if (cmd_id_list.length) {
                $.ajax({
                    type: 'POST',
                    url: 'index.php?route=extension/for_dealer_import/getJobsStatusByArrayId&token=' + token,
                    data: {'cmd_list': cmd_id_list},
                    dataType: 'json',
                    success: function (res) {
                        if (res.res == 0) {
                            console.debug(res);
                        } else {
                            $.each(res.data, function (cmd_id, value) {
                                let type = value.cmd;

                                $('span.' + type + '_status[data-cmd-id="' + cmd_id + '"]').html(statusTranslate[value.status_name] + ':');
                                $('span.' + type + '_records[data-cmd-id="' + cmd_id + '"]').html(value.records + ' (' + value.jobs + ')');

                                if (value.status_name === 'Finished' || value.status_name === 'Canceled') {
                                    $('.checkbox.update_dealer[data-cmd-id=' + cmd_id + ']').prop('checked', false);
                                    $('.checkbox.update_dealer[data-cmd-id=' + cmd_id + ']').attr('data-cmd-id', '');
                                }

                            });
                        }
                    }
                });
            }

        }


        function checkUnsavedReplaceChanges(element) {
            if ($(element).attr('data-json-replaces') !== $(element).attr('data-not-saved-json-replaces') &&
                $(element).attr('data-not-saved-json-replaces')
            ) {
                $('[replaces-modal-recover]').show();
            } else {
                $('[replaces-modal-recover]').hide();
            }
        }

        function getJsonForReplacesModal(element) {
            if (element) lastReplacesBtn = element;

            return JSON.parse($(element).attr('data-json-replaces') || '{}')
        }

        function addReplacesItem() {
            saveJsonReplaces();

            let json = JSON.parse($(lastReplacesBtn).attr('data-not-saved-json-replaces') || '{}');
            json[''] = '';
            buildReplacesModal(false, json);

            try {
                $('[replaces-modal-content]').find('[data-replaces-row]').last().find('[data-replace-from]').focus();
            } catch (e) {
            }
        }

        function removeReplaceItem(key) {
            let json = saveJsonReplaces();
            delete json[key];

            buildReplacesModal(false, json);
            saveJsonReplaces();
        }

        function onReplacesInputChange() {
            saveJsonReplaces();
        }

        function saveJsonReplaces() {
            let json = {};

            $('[replaces-modal-content]').find('[data-replaces-row]').each(function () {
                let key = $(this).find('[data-replace-from]').val().replace(/'/g, '');
                let val = $(this).find('[data-replace-to]').val().replace(/'/g, '');

                $(this).find('[data-replaces-key]').attr('data-replaces-key', key);

                json[key] = val;
            });

            $('[replaces-modal-recover]').hide();

            $(lastReplacesBtn).attr('data-not-saved-json-replaces', JSON.stringify(json));

            return json;
        }

        function buildReplacesModal(element, json = false) {

            if (element) {
                let parentRow = $(element).closest('[replaces-row-id]');
                $('#replaces-modal-title').text(
                    $(parentRow).find('.changed_option_name').val() || $(parentRow).find('.option_name').val() ||
                    'Свойство без имени :('
                );
            }

            let replaces = json || getJsonForReplacesModal(element);

            let replacesHtml = '';

            for (const replaceKey in replaces) {
                let replaceVal = replaces[replaceKey];

                replacesHtml += `
                <div class="col-12 d-flex w-100 m-0 p-0 mb-2" data-replaces-row="">                    
                    <div class="col-5 d-flex justify-content-center m-0 p-0">
                        <input class="w-100 form-control" data-replace-from="" value="${replaceKey}" 
                               style="min-width: 220px" placeholder="Что ищем" type="text"/>
                    </div>
                    <div class="col-1 h5 d-flex justify-content-center align-items-center m-0 p-0">
                        <span alt="f344" class="dashicons dashicons-arrow-right-alt"></span>
                    </div>
                    <div class="col-5 d-flex justify-content-center m-0 p-0">
                        <input class="w-100 form-control" value="${replaceVal}" data-replace-to="" 
                               style="min-width: 220px" placeholder="На что меняем" type="text"/>
                    </div>
                    <div class="col-1 h5 d-flex justify-content-center align-items-center m-0 p-0">
                        <span remove-replaces-item="" class="delete bg-warning pointer rounded-circle"
                              data-replaces-key="${replaceKey}"></span>
                    </div>
                </div>`;
            }

            $('[replaces-modal-content]').html(replacesHtml);
        }

        function recoverReplacesItems() {
            let json = $(lastReplacesBtn).attr('data-not-saved-json-replaces') || '{}';
            buildReplacesModal(false, JSON.parse(json));

            $('[replaces-modal-recover]').hide();
        }

        function saveReplaces(id, replaces) {

            let btn = lastReplacesBtn;
            let token = $('#user_token').val();

            let data = {
                id: id,
                replaces: replaces
            };

            $('[replaces-save-btn]').text('Сохраняю...');

            $.ajax({
                type: 'POST',
                url: 'index.php?route=extension/for_dealer_import/saveMergeReplaces&token=' + token,
                data: data,
                dataType: 'json',
                success: (res) => {

                    if (data.id != $(lastReplacesBtn).closest('[replaces-row-id]').attr('replaces-row-id')) {
                        return;
                    }

                    setTimeout(() => $('[replaces-save-btn]').text('Сохранить'), 1500);

                    if (res.res == 0) {
                        return console.debug(res);
                    }

                    $('[replaces-save-btn]').text('Сохранено!');
                    $(btn).attr('data-json-replaces', JSON.stringify(replaces));

                    $(`[data-count-replaces="${id}"]`).text(Object.keys(replaces).length);

                    buildReplacesModal(false, replaces);
                }
            });

        }


        setInterval(check_jobs_status, 5000);

        function notifyMsg(title, message) {
            notify({
                type: 'success', //alert | success | error | warning | info
                title: title,
                autoHide: true,
                delay: 3500,
                overlay: false,
                position: {
                    x: 'right', //right | left | center
                    y: 'bottom' //top | bottom | center
                },
                icon: '',
                message: message
            });
        }


        function combobox() {
            $.widget('custom.combobox', {
                _create: function () {
                    this.wrapper = $('<span>')
                        .addClass('custom-combobox')
                        .insertAfter(this.element);

                    this.element.hide();
                    this._createAutocomplete();
                    this._createShowAllButton();
                },

                _createAutocomplete: function () {
                    var selected = this.element.children(':selected'),
                        value = selected.val() ? selected.text() : '';

                    this.input = $('<input>')
                        .appendTo(this.wrapper)
                        .val(value)
                        .attr('title', '')
                        .addClass('custom-combobox-input ui-widget ui-widget-content ui-state-default ui-corner-left')
                        .autocomplete({
                            delay: 0,
                            minLength: 0,
                            source: $.proxy(this, '_source')
                        })

                    this._on(this.input, {
                        autocompleteselect: function (event, ui) {

                            ui.item.option.selected = true;
                            this._trigger('select', event, {
                                item: ui.item.option
                            });

                            $(this.element[0]).trigger('change');
                        },

                        autocompletechange: '_removeIfInvalid'
                    });
                },

                _createShowAllButton: function () {
                    var input = this.input,
                        wasOpen = false;

                    $('<a>')
                        .attr('tabIndex', -1)
                        .appendTo(this.wrapper)
                        .button({
                            icons: {
                                primary: 'ui-icon-triangle-1-s'
                            },
                            text: false
                        })
                        .removeClass('ui-corner-all')
                        .addClass('custom-combobox-toggle ui-corner-right')
                        .on('mousedown', function () {
                            wasOpen = input.autocomplete('widget').is(':visible');
                        })
                        .on('click', function () {
                            input.trigger('focus');

                            // Close if already visible
                            if (wasOpen) {
                                return;
                            }

                            // Pass empty string as value to search for, displaying all results
                            input.autocomplete('search', '');
                        });
                },

                _source: function (request, response) {
                    var matcher = new RegExp($.ui.autocomplete.escapeRegex(request.term), 'i');
                    response(this.element.children('option').map(function () {
                        var text = $(this).text();
                        if (this.value && (!request.term || matcher.test(text)))
                            return {
                                label: text,
                                value: text,
                                option: this
                            };
                    }));
                },

                _removeIfInvalid: function (event, ui) {

                    // Selected an item, nothing to do
                    if (ui.item) {
                        return;
                    }

                    // Search for a match (case-insensitive)
                    var value = this.input.val(),
                        valueLowerCase = value.toLowerCase(),
                        valid = false;
                    this.element.children('option').each(function () {
                        if ($(this).text().toLowerCase() === valueLowerCase) {
                            this.selected = valid = true;
                            return false;
                        }
                    });

                    // Found a match, nothing to do
                    if (valid) {
                        return;
                    }

                    // Remove invalid value
                    this.input
                        .val('')
                        .attr('title', value + ' didn\'t match any item')
                    this.element.val('');
                    this._delay(function () {
                        this.input.attr('title', '');
                    }, 2500);
                    this.input.autocomplete('instance').term = '';
                },

                _destroy: function () {
                    this.wrapper.remove();
                    this.element.show();
                }
            });

            $('[combobox]').not('[combobox="on"]').combobox();

            $('[combobox]').not('[combobox="on"]').attr('combobox', 'on').on('click', function () {
                $('[combobox]').toggle();
            });
        }

        combobox();

    });


})(jQuery);

window.queryChange = function (name, value) {
    const search = window.location.search;
    const params = new URLSearchParams(search);

    params.set(name, value);

    window.history.replaceState('', '', location.pathname + '?' + params.toString())
};

